/* Copyright (C) 1999 Free Software Foundation, Inc.
   This file is part of the GNU UTF-8 Library.

   The GNU UTF-8 Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU UTF-8 Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU UTF-8 Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */


/*
 * Microwindows/Nano-X utf-8 support.
 * This code has been taken from libutf-8 (http://clisp.cons.org/~haible/packages-libutf8.html)
 * by Bruno Haible <haible@clisp.cons.org>
 *
 * Martin Jolicoeur <martinj@visuaide.com> 2000
 *
 */


#ifndef _UTF8_WCHAR_H
#define _UTF8_WCHAR_H

/* Get FILE definition. */
#include <stdio.h>

/* Get errno declaration and values. */
#include <errno.h>
#ifndef EILSEQ
#define EILSEQ EINVAL
#endif

/* Constant expression of type `wint_t' whose value does not correspond
   to any member of the extended character set.  */
#ifndef WEOF
# define WEOF (0xffffffffu)
#endif

/* Define as the number of bits in the `wchar_t' type. */
#define WCHAR_T_BITS 16
/* Maximum length of a multibyte character.  */
#undef MB_CUR_MAX
#if WCHAR_T_BITS == 32
#define MB_CUR_MAX 32
#define MB_LEN_MAX 6
#else
#define MB_CUR_MAX 16
#define MB_LEN_MAX 3
#endif

#if (WCHAR_T_BITS == 16)
/* Use 16-bit Unicode. This does not cover 100% of the
   standards, but it is more space-saving than 32-bit Unicode.*/
#undef wchar_t
#define wchar_t utf8_wchar_t
typedef unsigned short wchar_t;
#else
/* Use 32-bit Unicode. This wastes a lot of space.*/
#undef wchar_t
#define wchar_t utf8_wchar_t
typedef unsigned int wchar_t;
#endif

/* Integral type unchanged by default argument promotions that can
   hold any value corresponding to members of the extended character
   set, as well as at least one value that does not correspond to any
   member of the extended character set.  */
#undef wint_t
#define wint_t utf8_wint_t
typedef unsigned int wint_t;

#undef WCHAR_MIN
#undef WCHAR_MAX
#define WCHAR_MIN ((wchar_t) 0)
#define WCHAR_MAX (~WCHAR_MIN)

/* Unsigned version of the `wchar_t' type. */
#define u_wchar_t wchar_t

#ifdef __cplusplus
extern "C"
{
#endif

/* Write wide character representation of multibyte character pointed to by S
   to PWC. */
  extern size_t utf8_mbrtowc (wchar_t * pwc, const char *s, size_t n);

/* Write multibyte representation of wide character WC to S. */
  extern size_t utf8_wcrtomb (char *s, wchar_t wc);

/* Return number of bytes in multibyte character pointed to by S. */
  extern size_t utf8_mbrlen (const char *s, size_t n);

/* Write wide character representation of multibyte character string SRC
   to DEST. */
  extern size_t utf8_mbsrtowcs (wchar_t * dest, const char **src, size_t len);

/* Write multibyte character representation of wide character string SRC
   to DEST. */
  extern size_t utf8_wcsrtombs (char *dest, const wchar_t ** src, size_t len);

/* GNU extension. */
/* Write wide character representation of at most NMC bytes of the multibyte
   character string SRC to DEST. */
  extern size_t utf8_mbsnrtowcs (wchar_t * dest, const char **src, size_t nms,
				 size_t len);

/* GNU extension. */
/* Write multibyte character representation of at most NWC characters from
   the wide character string SRC to DEST. */
  extern size_t utf8_wcsnrtombs (char *dest, const wchar_t ** src, size_t nwc,
				 size_t len);


/* Reads the next wide character from STREAM, and returns it. Returns WEOF
   when end-of-file is reached or an error is encountered. */
  extern wint_t utf8_fgetwc (FILE * stream);

/* Reads the next wide character from stdin, and returns it. Returns WEOF
   when end-of-file is reached or an error is encountered. */
  extern wint_t utf8_getwchar (void);

/* Reads a line of wide characters from STREAM. WS points to a buffer for
   N wide characters. At most N-1 wide characters will be read, stopping
   after the first seen newline character. A null wide character will be
   appended. Then WS is returned. If no wide character could be read from
   STREAM, or if an error is encountered, NULL is returned.
   Note: This function is unreliable, because it does not permit to deal
   properly with null wide characters that may be present in the input. */
  extern wchar_t *utf8_fgetws (wchar_t * ws, int n, FILE * stream);

/* Pushes back a wide character onto STREAM. Returns WC if successful, or
   WEOF if WC is WEOF or when an error is encountered. */
  extern wint_t utf8_ungetwc (wint_t wc, FILE * stream);

#define utf8_putwc utf8_fputwc
#define utf8_getwc utf8_fgetwc

/* Writes WC to STREAM. Returns WC if successful, or WEOF when an error is
   encountered. */
  extern wint_t utf8_fputwc (wchar_t wc, FILE * stream);

/* Writes WC to stdout. Returns WC if successful, or WEOF when an error is
   encountered. */
  extern wint_t utf8_putwchar (wchar_t wc);

/* Writes the contents of the wide string WS, up to but not including the
   terminating null wide character, to STREAM. Returns a nonnegative integer
   if successful, or -1 in case of error. */
  extern int utf8_fputws (const wchar_t * ws, FILE * stream);

/* Return the length of the multibyte character in S, which is no longer
   than N. */
  extern int utf8_mblen (const char *s, size_t n);

/* Return the length of the given multibyte character, putting its `wchar_t'
   representation in *PWC. */
  extern int utf8_mbtowc (wchar_t * pwc, const char *s, size_t n);

/* Put the multibyte character represented by WCHAR in S, returning its
   length. */
  extern int utf8_wctomb (char *s, wchar_t wc);

/* Convert a multibyte string to a wide char string. */
  extern size_t utf8_mbstowcs (wchar_t * pwcs, const char *s, size_t n);

/* Convert a wide char string to multibyte string. */
  extern size_t utf8_wcstombs (char *s, const wchar_t * pwcs, size_t n);

#ifdef __cplusplus
}
#endif

#endif				/* _UTF8_STDLIB_H */
